#!/usr/bin/env python

import omero.clients
from omero.rtypes import unwrap

from prometheus_client import (
    Gauge,
    Summary,
)
import yaml


def connect(hostname, username, password):
    client = omero.client(hostname)
    client.setAgent('prometheus-omero-counts')
    client.createSession(username, password)
    return client


SESSION_REQUEST_TIME = Summary(
    'omero_counts_processing_seconds', 'Time spent counting sessions')


class QueryMetric(object):
    def __init__(self, name, cfg, verbose):
        self.name = name
        description = cfg['description']
        self.query = cfg['query']
        labels = cfg['labels']
        # Let prometheus_client handle name validation
        self.prometheus_gauge = Gauge(self.name, description, labels)
        self.verbose = verbose
        # HQL count won't return 0 so need to explicitly delete labelsets
        self.labelsets = set()

    def update(self, queryservice):
        results = queryservice.projection(
            self.query, None, {'omero.group': '-1'})
        if not results:
            if self.verbose:
                print('%s NULL' % self.name)
        prev_labelsets = self.labelsets
        self.labelsets = set()
        for r in results:
            labelvalues = [lv for lv in unwrap(r[1:])]
            value = unwrap(r[0])
            self.prometheus_gauge.labels(*labelvalues).set(value)
            self.labelsets.add(tuple(labelvalues))
            if self.verbose:
                print('%s %s %s' % (self.name, labelvalues, value))
        # Now delete absent labelsets
        for rm in prev_labelsets.difference(self.labelsets):
            self.prometheus_gauge.remove(*rm)
            if self.verbose:
                print('Removed %s %s' % (self.name, rm))


class CountMetrics(object):

    def __init__(self, client, configfiles, verbose=False):
        self.client = client
        self.verbose = verbose

        self.metrics = {}
        for f in configfiles:
            with open(f) as fh:
                cfg = yaml.load(fh)
            for name in cfg:
                if name in self.metrics:
                    raise Exception(
                        'Duplicate metric name found: %s' % name)
                self.metrics[name] = QueryMetric(name, cfg[name], verbose)

    @SESSION_REQUEST_TIME.time()
    def update(self):
        queryservice = self.client.getSession().getQueryService()
        for metric in self.metrics.values():
            metric.update(queryservice)



#!/usr/bin/env python

import collections
import omero.clients
import omero.cmd
from omero.rtypes import unwrap

from prometheus_client import (
    Gauge,
    Summary,
)


def connect(hostname, username, password):
    client = omero.client(hostname)
    client.setAgent('prometheus-omero-sessions')
    client.createSession(username, password)
    return client


SESSION_REQUEST_TIME = Summary(
    'omero_sessions_processing_seconds', 'Time spent counting sessions')


class SessionMetrics(object):

    lastusers = set()
    g_sessions = Gauge(
        'omero_sessions_active', 'Active OMERO sessions', ['username'])
    g_users_total = Gauge('omero_users_total', 'Number of OMERO users',
                          ['active'])
    g_groups_total = Gauge('omero_groups_total', 'Number of OMERO groups')

    def __init__(self, client, verbose=False):
        self.client = client
        self.verbose = verbose

    @SESSION_REQUEST_TIME.time()
    def update(self):
        # https://github.com/openmicroscopy/openmicroscopy/blob/v5.4.0-m1/components/tools/OmeroPy/src/omero/plugins/sessions.py#L714

        cb = None
        try:
            cb = self.client.submit(
                omero.cmd.CurrentSessionsRequest(), 60, 500)
            rsp = cb.loop(60, 500)
            counts = collections.Counter(c.userName for c in rsp.contexts)
            missing = self.lastusers.difference(counts.keys())
            for m in missing:
                if self.verbose:
                    print('%s: %d' % (m, 0))
                self.g_sessions.labels(m).set(0)
            for username, n in counts.items():
                if self.verbose:
                    print('%s: %d' % (username, n))
                self.g_sessions.labels(username).set(n)
                self.lastusers.add(username)
        except omero.CmdError as ce:
            # Unwrap the CmdError due to failonerror=True
            raise Exception(ce.err)
        finally:
            if cb:
                cb.close(True)

        adminservice = self.client.getSession().getAdminService()

        user_group_id = adminservice.getSecurityRoles().userGroupId
        users_active = 0
        users_inactive = 0
        for user in adminservice.lookupExperimenters():
            if user_group_id in (unwrap(g.getId()) for g in
                                 user.linkedExperimenterGroupList()):
                users_active += 1
            else:
                users_inactive += 1
        self.g_users_total.labels(1).set(users_active)
        self.g_users_total.labels(0).set(users_inactive)

        group_count = len(adminservice.lookupGroups())
        self.g_groups_total.set(group_count)

        if self.verbose:
            print('Users (active/inactive): %d/%d' % (
                users_active, users_inactive))
            print('Groups: %d' % group_count)



#!/usr/bin/env python

import argparse
import omero.clients

from prometheus_client import (
    Gauge,
    start_http_server,
)
from time import (
    sleep,
    time,
)
from omero_prometheus_tools import (
    CountMetrics,
    SessionMetrics,
)


def connect(hostname, username, password):
    client = omero.client(hostname)
    client.setAgent('prometheus-omero-tools')
    client.createSession(username, password)
    return client


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--config', action='append',
                        help='Query configuration files')
    parser.add_argument('-s', '--host', default='localhost')
    parser.add_argument('-u', '--user', default='guest')
    parser.add_argument('-w', '--password', default='guest')
    parser.add_argument('-l', '--listen', type=int, default=9449,
                        help='Serve metrics on this port')
    parser.add_argument('-i', '--interval', type=int, default=60,
                        help='Interval (seconds) between updates, default 60')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Print verbose output')
    args = parser.parse_args()

    g_last_login = Gauge('omero_prometheus_tools_agent_login_time',
                         'Time of last Prometheus agent login')

    client = connect(args.host, args.user, args.password)
    # Don't catch exception, exit on login failure so user knows
    g_last_login.set_to_current_time()

    try:
        # Start up the server to expose the metrics.
        start_http_server(args.listen)
        if args.config:
            counts = CountMetrics(client, args.config, args.verbose)
        else:
            counts = None
        sessions = SessionMetrics(client, verbose=args.verbose)
        while True:
            starttm = time()
            sessions.update()
            if counts:
                counts.update()
            endtm = time()
            # HQL queries may take a long time
            sleep(max(args.interval + endtm - starttm, 0))
    finally:
        client.closeSession()



from setuptools import setup
setup(
    name='omero-prometheus-tools',
    version='0.2.0',
    description='Tools for monitoring OMERO with Prometheus',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    scripts=[
        'omero_prometheus_tools/omero-prometheus-tools.py',
    ],
    data_files=[
        ('etc', [
            'etc/prometheus-omero-counts.yml',
        ]),
    ],
    packages=[
        'omero_prometheus_tools',
    ],
    install_requires=[
        'omero-py>=5.6.0',
        # TODO: Update to current release 0.7.*
        'prometheus-client==0.2.*',
        'PyYAML',
    ],
    python_requires='>=3',
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Monitoring',
    ],
    # Allow external access to etc files
    zip_safe=False,
)
